/** @file store_htable.h
 *
 * @brief Simple in-memory triple store back end based on hash tables.
 *
 * This is the simplest choice to do in-memory manipulation of RDF graphs and
 * it has some limitations: most notably, it only supports triples without
 * context (one graph per store) and it is not indexed. This means that it is
 * optimized for fast writes and sequential lookups (iteration). Lookups on
 * arbitrary terms are supported but require iterating over all the triples.
 * This implementation is most convenient for graphs where retrieval is done
 * via iteration.
 *
 * Also, as it may be obvious, this store is not persistent.
 *
 * For faster random lookups and persistence, the MDB backend is preferred. If
 * persistence is not required (e.g. ingesting and manipulating a very large
 * graph and outputting some results on a file) an ad-hoc MDB store located in
 * RAM disk can be used, which is much faster.
 */
#ifndef _LSUP_STORE_HTABLE_H
#define _LSUP_STORE_HTABLE_H

#include "buffer.h"
#include "store_interface.h"


extern const LSUP_StoreInt htstore_int;

#endif  // _LSUP_STORE_HTABLE_H
